﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using MoNo;
using MoNo.OpenGL;
using MoNo.Studio;
using MoNo.CSharpUtils;

namespace QuickCsTemplate
{
  [Export( typeof( MoNo.Studio.IMenuFactory ) )]
  class OpenGLMenuFactory : MenuFactoryForCSharp<QuickAppContext>
  {
    public override IEnumerable<MenuItem> EnumerateMenuItems( QuickAppContext con )
    {
      yield return MenuItemFactory.Folder( "Samples",
        MenuItemFactory.Folder( "OpenGL Samples",

        // OpenGL を P/Invoke で直接呼び出して直線を描画
        con.CreateCommand( () =>
        {
          // MoNo.Graphics.IScene のインスタンスを生成
          var scene = GraphicsUT.CreateScene( sc =>
          {
            // MoNo.OpenGL.GL クラスを利用すると P/Invoke により直接 OpenGL の API を呼び出すことができます。
            GL.glDisable( GL.GL_LIGHTING );  // ライティングを無効化
            GL.glColor3f( 1, 1, 0 );   // 色を設定
            GL.glBegin( GL.GL_LINES );
            GL.glVertex3f( 0, 0, 0 );
            GL.glVertex3f( 1, 0, 0 );
            GL.glEnd();
          } );
          con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) ); // シーンオブジェクトを登録
        } ).ToMenuItem( "OpenGL Direct Call" ),

        // MGL を利用して直線を描画
        con.CreateCommand( () =>
        {
          // MoNo.Graphics.IScene のインスタンスを生成
          var scene = GraphicsUT.CreateScene( sc =>
          {
            MGL.LightingEnabled = false;
            MGL.Color = System.Drawing.Color.Pink.ToColor4f();
            using ( var gl = MGL.Begin( GLPrimType.Lines ) ) {
              gl.Vertex( Point3d.Zero );
              gl.Vertex( new Point3d( 1.0, 1.0, 1.0 ) );
            }
          } );
          con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) ); // シーンオブジェクトを登録
        } ).ToMenuItem ("By MoNo.OpenGL.MGL" ),

        // ISceneContext を利用して直線を描画
        con.CreateCommand( () =>
        {
          // MoNo.Graphics.IScene インターフェイスのインスタンスを生成します
          var scene = GraphicsUT.CreateScene( sc =>
          {
            using ( var scope = sc.Push() ) { // scope を利用すると Lighting 等の状態変更が using スコープ内に限定されます
              scope.Lighting = false;
              scope.Color = System.Drawing.Color.Plum;
              sc.DrawLines( gl => gl.Vertices( Point3d.Zero, new Point3d( 1.0, 2.0, 3.0 ) ) );
            }
          } );
          con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) ); // シーンオブジェクトを登録
        } ).ToMenuItem( "By MoNo.Graphics.ISceneContext" ),

        // 頂点配列による折線描画（MGL）
        con.CreateCommand( () =>
        {
          // 頂点配列を作成
          var array = new[]
          {
            new Point3d( 0.0, 0.0, 0.0 ),
            new Point3d( 0.2, 0.0, 0.0 ),
            new Point3d( 0.2, 0.2, 0.0 ),
            new Point3d( 0.0, 0.2, 0.0 ),
            new Point3d( 0.0, 0.4, 0.0 ),
            new Point3d( 0.2, 0.4, 0.0 ),
            new Point3d( 0.2, 0.6, 0.0 ),
            new Point3d( 0.0, 0.6, 0.0 ),
          };
          // MoNo.Graphics.IScene のインスタンス
          var scene = GraphicsUT.CreateScene( sc =>
          {
            using ( var scope = sc.Push() ) { // scope を利用すると Lighting 等の状態変更が using スコープ内に限定されます
              scope.Lighting = false;
              scope.Color = System.Drawing.Color.Salmon;
              MGL.DrawArrays( GLPrimType.LineStrip, array );
            }
          } );
          // シーンオブジェクトを登録
          con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) ); // シーンオブジェクトを登録
        } ).ToMenuItem( "VertexArray (MoNo.OpenGL.MGL)" ),

        // 頂点配列による折線描画（GraphicsUT）
        con.CreateCommand( () =>
        {
          // 頂点配列を作成
          var array =
            GraphicsUT.CreateScene( GLPrimType.LineStrip, new[]
            {
              new Point3d( 0.0, 0.0, 0.0 ),
              new Point3d( 0.2, 0.0, 0.0 ),
              new Point3d( 0.2, 0.2, 0.0 ),
              new Point3d( 0.0, 0.2, 0.0 ),
              new Point3d( 0.0, 0.4, 0.0 ),
              new Point3d( 0.2, 0.4, 0.0 ),
              new Point3d( 0.2, 0.6, 0.0 ),
              new Point3d( 0.0, 0.6, 0.0 ),
            } );
          // MoNo.Graphics.IScene のインスタンス
          var scene = GraphicsUT.CreateScene( sc =>
          {
            using ( var scope = sc.Push() ) { // scope を利用すると Lighting 等の状態変更が using スコープ内に限定されます
              scope.Lighting = false;
              scope.Color = System.Drawing.Color.Salmon;
              array.Draw( sc ); // 頂点配列シーンの呼び出し
            }
          } );
          // シーンオブジェクトを登録
          con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) );
        } ).ToMenuItem( "VertexArray (MoNo.GraphicsUT)" ),

        // テクスチャマッピング
        con.CreateCommand( () =>
          con.ShowOpenFileDialog( "画像ファイル (*.bmp;*.jpg;*.png)|*.bmp;*.jpg;*.png" ).Do( path =>
          {
            // 画像ファイルの読込
            var bitmap = new System.Drawing.Bitmap( System.Drawing.Image.FromFile( path ) );

            // テクスチャオブジェクトを生成します。
            // ※ 注意！！ OpenGL 2.0 未満の場合はテクスチャのサイズは 2 のべき乗でなくてはなりません。
            //            詳しくは OpenGL の仕様を参照して下さい。
            var texture = MGL.CreateTextureObject( bitmap );

            // シーングラフにテクスチャを描画するシーンを登録します。
            var scene = GraphicsUT.CreateScene( sc =>
              {
                using ( var scope = sc.Push() ) { // scope を利用すると Lighting 等の状態変更が using スコープ内に限定されます
                scope.Lighting = false;
                  scope.Color = System.Drawing.Color.White;
                  using ( var binding = texture.Bind() ) {
                    sc.DrawQuads( gl =>
                  {
                    gl.TexCoord( 0.0, 0.0 ); gl.Vertex( 0.0, 0.0, 0.0 );
                    gl.TexCoord( 1.0, 0.0 ); gl.Vertex( 1.0, 0.0, 0.0 );
                    gl.TexCoord( 1.0, 1.0 ); gl.Vertex( 1.0, 1.0, 0.0 );
                    gl.TexCoord( 0.0, 1.0 ); gl.Vertex( 0.0, 1.0, 0.0 );
                  } );
                  }
                }
              } );
            // シーンオブジェクトを登録
            con.Document.Val.Entries.Add( new MoNo.Wpf.Entry( scene ) );
          } ) ).ToMenuItem( "Texture Mapping" )
        )
      );
    }
  }
}
